#ifndef CFQ_APPENDER_H
#define CFQ_APPENDER_H

#include <string>
#include <vector>

#include "plog/file_appender.h"

namespace plog
{
    
    //////////////////////////////////////////////////////////////////////////

    // Circular File Queue (CFQ) appender

    // CFQ is a set of files of limited size. There are two limits - maximum
    // file size and maximum number of files. When file size reaches its limit 
    // , appender starts writing into the next file. When file number reaches 
    // its limit, appender deletes the oldest file before starting to write to
    // the newest one.

    class cfq_appender: public appender
    {
    public:
        cfq_appender(log_level min_severity = none, 
            size_t max_file_size = 10 * 1024 * 1024, 
            size_t max_files = 2);

        cfq_appender(const wchar_t* file_name_base, 
            const wchar_t* file_extension,
            size_t max_file_size, 
            size_t max_files,
            log_level min_severity = none);

        void init(const wchar_t* file_name_base, const wchar_t* file_extension);

        void write(const void* buffer, size_t length);
        void write_header(const void* buffer, size_t length);

    private:
        void start_new_file();
        void continue_file();
        void write_header();

    private:
        std::wstring file_name_base_;
        std::wstring file_extension_;
        size_t file_index_;
        size_t file_count_;
        size_t max_file_size_;
        size_t max_files_;
        file_appender file_appender_;
        std::vector<char> header_;
    };

    //////////////////////////////////////////////////////////////////////////

    namespace detail
    {
        void format_file_name(const std::wstring& base, 
            const std::wstring& extension, 
            size_t number, 
            std::wstring* file_name);
    }
}

#endif