#ifndef PLOG_INIT_H
#define PLOG_INIT_H

#include <memory>
#include <ostream>

#include "plog/log_level.h"
#include "plog/formatter.h"
#include "plog/custom_log.h"

namespace plog
{
    class logger;

    struct initializer
    {
        virtual ~initializer(){}

        virtual logger* create() = 0;
        virtual void destroy(logger* l) = 0;
    };

    void init(std::auto_ptr<initializer> li);//may throw
    
    void init(const wchar_t* filename, 
        const log_level level, 
        bool rewrite_file = false);

    void init(const wchar_t* filename, 
        const log_level level, 
        size_t max_file_size, 
        size_t max_files);
    
    void init_csv(const wchar_t* filename, const log_level level, bool rewrite_file = false);
    void init_csv(const wchar_t* filename, const log_level level, size_t max_file_size, size_t max_files);
    void init_txt(const wchar_t* filename, const log_level level, bool rewrite_file = false);
    void init_txt(const wchar_t* filename, const log_level level, size_t max_file_size, size_t max_files);

    void init_ostream(std::ostream& stream, const log_level level, plog::formatter* fmtr = NULL);
    void init_cerr(const log_level level);
    void init_debug(const log_level level);

    void init_custom_log(custom_log* log);

    void free(bool free_resources = true);

    namespace detail
    {
        void parse_file_name(const wchar_t* filename, std::wstring& base, std::wstring& extension);
        bool is_csv(const wchar_t* filename);
    }
}

#endif
