#ifndef PLOG_LOG_H
#define PLOG_LOG_H

#include "plog/config.h"
#include "plog/version.h"

#include "plog/logger_holder.h"
#include "plog/log_message.h"
#include "plog/logger.h"
#include "plog/init.h"

/* We need this nesting to let __LINE__ to be interpreted by preprocessor*/
#define PLOG_VAR_IMPL(base, line) base ## line

#define PLOG_VAR(base, line) PLOG_VAR_IMPL(base, line)

#define PLOG_THIS PLOG_VAR(plog_this, __LINE__)

#define PLOG_COUNTER PLOG_VAR(plog_counter, __LINE__)

#ifdef _MSC_BUILD
#define PLOG_RETRIEVE_OBJ(var)\
    const void* var = NULL;\
    __if_exists(this) \
    {\
        var = this;\
    }
#else
#define PLOG_RETRIEVE_OBJ(var)\
    const void* var = NULL;
#endif

//////////////////////////////////////////////////////////////////////////

#define LOG(severity) \
    PLOG_RETRIEVE_OBJ(PLOG_THIS)\
    if (plog::logger_holder::get_logger(severity))\
    *(plog::logger_holder::get_logger()) += plog::message(severity, CONTEXT_INFO(PLOG_THIS), plog::logger_holder::get_logger()->get_locale())

#define LOG_IF(severity, condition) \
    PLOG_RETRIEVE_OBJ(PLOG_THIS)\
    if (condition && plog::logger_holder::get_logger(severity))\
    *(plog::logger_holder::get_logger()) += plog::message(severity, CONTEXT_INFO(PLOG_THIS), plog::logger_holder::get_logger()->get_locale())

#define LOG_EVERY_N(severity, n) \
    static int PLOG_COUNTER = 0; \
    ++PLOG_COUNTER; \
    LOG_IF(severity, (PLOG_COUNTER % n == 0))

#define LOG_IF_EVERY_N(severity, condition, n) \
    static int PLOG_COUNTER = 0; \
    ++PLOG_COUNTER; \
    LOG_IF(severity, (condition && (PLOG_COUNTER % n == 0)) )

#define LOG_FIRST_N(severity, n) \
    static int PLOG_COUNTER = 0; \
    ++PLOG_COUNTER; \
    LOG_IF(severity, (PLOG_COUNTER <= n))

#endif
