#ifndef PLOG_LOG_MESSAGE_H
#define PLOG_LOG_MESSAGE_H

#include <ctime>
#include <locale>

#include <windows.h>

#include "plog/log_message_stream.h"
#include "plog/log_level.h"
#include "plog/context_info.h"

namespace plog
{

#define CONTEXT_INFO(OBJ) plog::context_info(__FILE__, __FUNCTION__, __LINE__,::GetCurrentThreadId(), OBJ)

    //////////////////////////////////////////////////////////////////////////

    class message
    {
        friend bool operator==(const message& msg1, const message& msg2);

    public:
        message(log_level severity, const context_info& context, const std::locale& loc, const char* message = "")
            : severity_(severity)
            , stream_(loc, message)
            , context_(context)
        {
        }

        ~message()
        {

        }

        template<typename T>
        message& operator<< (const T& chunk)
        {
            stream_ << chunk;
            return *this;
        }

        std::string text() const
        {
            return stream_.str();
        }

        const char* first() const
        {
            return stream_.first();
        }

        const char* last() const
        {
            return stream_.last();
        }

        size_t size() const
        {
            return stream_.size();
        }

        const char* severity_name() const
        {
            return level_name(severity_);
        }

        log_level severity() const
        {
            return severity_;
        }

        const context_info& context() const
        {
            return context_;
        }

    private:
        message_stream stream_;
        log_level severity_;
        context_info context_;
    };

    inline bool operator==(const message& msg1, const message& msg2)
    {
        if (msg1.severity_ == msg2.severity_ && msg1.stream_.str() == msg2.stream_.str())
            return true;
        else
            return false;
    }

    inline std::ostream& operator<< (std::ostream &stream, const message& msg)
    {
        stream << msg.severity_name() << ":" << msg.text();
        return stream;
    }
}

#endif