import os, subprocess, sys

from string import Template

tpl = '''\
#ifndef PLOG_VERSION_H
#define PLOG_VERSION_H

//////////////////////////////////////////////////////////////////////////
//
// This file is generated automatically. Do not modify it
//
//////////////////////////////////////////////////////////////////////////

#define PLOG_MAJOR_VERSION  ${vermajor}
#define PLOG_MINOR_VERSION  ${verminor}
#define PLOG_PATCH_VERSION  ${verpatch}

//////////////////////////////////////////////////////////////////////////

#define PLOG_STRINGIZE_IMPL(num) #num
#define PLOG_S(macronum) PLOG_STRINGIZE_IMPL(macronum)

//////////////////////////////////////////////////////////////////////////

#define PLOG_VERSION_STRING "" \\
    PLOG_S(PLOG_MAJOR_VERSION) "." \\
    PLOG_S(PLOG_MINOR_VERSION) "." \\
    PLOG_S(PLOG_PATCH_VERSION)

namespace plog
{
    inline const char* get_version()
    {
        return PLOG_VERSION_STRING;
    }
}

#endif
'''

def fixversion(major, minor, patch):
    f = open('include/plog/version.h', 'w')
    s = Template(tpl).substitute(vermajor=str(major), verminor=str(minor), verpatch=str(patch))
    f.write(s)
    f.close()
    
def getlastversion():
    p = subprocess.Popen('hg tags', stdout=subprocess.PIPE)
    ret = p.wait()
    if ret != 0:
        raise RuntimeError

    lines = p.stdout.readlines()
    tags = [x.split(' ')[0] for x in lines]
    versions = [x for x in tags if x.count('.') == 2]

    latest = [int(x) for x in max(versions).split('.')]

    return tuple(latest)

def increment(major, minor, patch, what):

    if what == 'major':
        return (major+1, 0, 0)

    if what == 'minor':
        return (major, minor+1, 0)

    if what == 'patch':
        return (major, minor, patch+1)

    raise RuntimeError('WTF? Don\'t know how to increment ' + what)

def branch():
    return subprocess.check_output('hg branch').strip()

def update(rev):
    ret = subprocess.call('hg update ' + rev)
    if ret != 0:
        raise RuntimeError

def merge(rev):
    ret = subprocess.call('hg merge -r ' + rev)
    if ret != 0:
        raise RuntimeError

def commit(message):
    ret = subprocess.call('hg commit -m "' + message + '"')
    if ret != 0:
        raise RuntimeError

def tag(rev, major, minor, patch):
    ret = subprocess.call('hg tag ' + str(major) + '.' + str(minor) + '.' + str(patch) + ' -r ' + rev)
    if ret != 0:
        raise RuntimeError

def edit_changelog(major, minor, patch):
    f = open('CHANGELOG', 'r')
    content=  f.readlines()
    f.close()

    content.insert(10, '\n')
    content.insert(10, '\n')
    content.insert(10, 'version ' + str(major) + '.' + str(minor) + '.' + str(patch) + '\n')

    f = open('CHANGELOG', 'w')
    f.writelines(content)
    f.close()

    ret = subprocess.call('notepad.exe CHANGELOG')
    if ret != 0:
        raise RuntimeError

def main(argv):
    if len(argv) == 0:
        inctarget = 'patch'
    else:
        inctarget = argv[0]

    (major, minor, patch) = getlastversion()
    (major, minor, patch) = increment(major, minor, patch, inctarget)

    current_branch = branch()
    update('stable')
    merge (current_branch)
    commit(current_branch + '=>stable')

    fixversion(major, minor, patch)
    edit_changelog(major, minor, patch)
    commit('version number updated')

    update(current_branch)
    tag('stable', major, minor, patch)



################################################################


if __name__ == "__main__":
    main(sys.argv[1:])
