#define _CRT_SECURE_NO_WARNINGS

#include "plog/cfq_appender.h"
#include "file_utils.h"

plog::cfq_appender::cfq_appender(log_level min_severity,
                                 size_t max_file_size, 
                                 size_t max_files)
: appender(min_severity)
, file_appender_(min_severity)
, file_index_(0)
, file_count_(0)
, max_file_size_(max_file_size)
, max_files_(max_files)
{

}

plog::cfq_appender::cfq_appender(const wchar_t* file_name_base, 
                                 const wchar_t* file_extension, 
                                 size_t max_file_size, 
                                 size_t max_files, 
                                 log_level min_severity)
: appender(min_severity)
, file_appender_(min_severity)
, file_index_(0)
, file_count_(0)
, max_file_size_(max_file_size)
, max_files_(max_files)
{
    init(file_name_base, file_extension);
}

void plog::cfq_appender::init(const wchar_t* file_name_base, const wchar_t* file_extension)
{
    file_name_base_ = file_name_base;
    file_extension_ = file_extension;
    start_new_file();
}

void plog::cfq_appender::start_new_file()
{
    std::wstring file_name;
    detail::format_file_name(file_name_base_, file_extension_, file_index_++, &file_name);
    file_appender_.init(file_name.c_str(), true);
    ++file_count_;
}

void plog::cfq_appender::write(const void* buffer, size_t length)
{
    if (length + file_appender_.file_size() > max_file_size_)
    {
        start_new_file();

        if (file_count_ > max_files_)
        {
            std::wstring deletion_candidate;
            detail::format_file_name(file_name_base_, file_extension_, file_index_ - max_files_ - 1, &deletion_candidate);
            if (detail::is_file_exists(deletion_candidate.c_str()))
            {
                detail::delete_file(deletion_candidate.c_str());
                --file_count_;
            }
        }

        write_header();
    }

    file_appender_.write(buffer, length);
}

void plog::cfq_appender::write_header(const void* buffer, size_t length)
{
    header_.assign(reinterpret_cast<const char*>(buffer), reinterpret_cast<const char*>(buffer) + length);
    write_header();
}

void plog::cfq_appender::write_header()
{
    if (header_.size())
    {
        file_appender_.write(&header_.front(), header_.size());
    }
}

void plog::detail::format_file_name(const std::wstring& base, const std::wstring& extension, size_t number, std::wstring* file_name)
{
    wchar_t numstr[] = L"000";
    _snwprintf(numstr, 3, L"%03d", number % 1000);
    *file_name = base + numstr;

    if (extension.size() && extension[0] != L'.')
    {
        file_name->push_back(L'.');
    }
    file_name->append(extension);
}