#include "plog/csv_formatter.h"
#include "plog/str_utils.h"

#include <algorithm>
#include <windows.h>
#include <cstdlib>

namespace
{
    const char CSV_HEADER[] = "Date;Time;Severity;TID;File;This;Function;Message"NEWLINE;

    int format_csv_item(char* buffer, size_t max_size, const plog::context_info& context, const char* severity_name, const char* text, size_t length)
    {
        const char* file_name = plog::detail::base_name(context.file);
        int res = _snprintf_s(buffer, max_size, _TRUNCATE, "%04d/%02d/%02d;%02d:%02d:%02d:%03d;%s;%d;%s@%d;0x%p;%s;\"%.*s\""NEWLINE,
            context.time.wYear, context.time.wMonth, context.time.wDay, 
            context.time.wHour, context.time.wMinute, context.time.wSecond, context.time.wMilliseconds,
            severity_name, context.tid, file_name, context.line, context.object, context.func_name, 
            length, text);
        return res;
    }

    size_t calculate_length(int sprintf_result, size_t prev_length, size_t max_size)
    {
        if (sprintf_result == -1)
        {
            return max_size - 1;
        }
        else
        {
            return prev_length + sprintf_result;
        }
    }

    void normalize(int sprintf_result, size_t length, char* buffer)
    {
        if (sprintf_result == -1)
        {
            buffer[length - 2] = '\r';
            buffer[length - 1] = '\n';
        }
    }
}

size_t plog::csv_formatter::format(const message& msg, char* buffer, size_t buffer_size) const
{
    int res = format_csv_item(buffer, buffer_size, msg.context(), msg.severity_name(), msg.first(), msg.size());
    size_t len = calculate_length(res, 0, buffer_size);
    normalize(res, len, buffer);

    return len;
}

size_t plog::csv_formatter::header_size() const
{
    return sizeof(CSV_HEADER) - sizeof(char);
}

const char* plog::csv_formatter::header() const
{
    return CSV_HEADER;
}
