#include "plog/multiappender_logger.h"
#include "plog/file_appender.h"
#include "plog/log_message.h"

#include <algorithm>

void plog::multiappender_logger::write(const message& msg)
{
    for (std::vector< std::pair<appender*,formatter*> >::iterator it = appenders_.begin();
        it != appenders_.end();
        ++it)
    {
        if (it->first->get_level() >= msg.severity())
        {
            const size_t BUF_SIZE = 512;
            char buffer[BUF_SIZE];

            size_t length = it->second->format(msg, buffer, BUF_SIZE);
            it->first->write(buffer, length);
        }
    }
}

bool plog::multiappender_logger::check_severity(log_level severity)
{
    std::vector< std::pair<appender*,formatter*> >::const_iterator it = 
        std::min_element(appenders_.begin(), appenders_.end());

    if (it == appenders_.end() || (*it).first->get_level() < severity)
        return false;

    return true;
}

void plog::multiappender_logger::add_appender(appender* log_appender, formatter* log_formatter)
{
    appenders_.push_back( std::pair<appender*, formatter*>(log_appender, log_formatter) );
    log_appender->write_header(log_formatter->header(), log_formatter->header_size());
}
