#include "plog/txt_formatter.h"
#include "plog/str_utils.h"

#include <algorithm>
#include <windows.h>
#include <cstdlib>

namespace 
{
    size_t find_eol(const char* p, const char* e, size_t* eol_pos)
    {
        size_t size = 0;
        for (;p + size != e; ++size)
        {
            if (p[size] == '\x0A')
            {
                *eol_pos = size;
                if (size > 1 && p[size-1] == '\x0D')
                    --(*eol_pos);

                return size + 1;
            }
        }

        *eol_pos = size;
        return size;
    }

    size_t calculate_length(int sprintf_result, size_t prev_length, size_t max_size)
    {
        if (sprintf_result == -1)
        {
            return max_size - 1;
        }
        else
        {
            return prev_length + sprintf_result;
        }
    }

    void normalize(int sprintf_result, size_t length, char* buffer)
    {
        if (sprintf_result == -1)
        {
            buffer[length - 2] = '\r';
            buffer[length - 1] = '\n';
        }
    }

    int format_text_line(char* buffer, size_t max_size, const plog::context_info& context, const char* severity_name, const char* text, size_t length)
    {
        int res = _snprintf_s(buffer, max_size, _TRUNCATE, "%04d.%02d.%02d\t%02d:%02d:%02d:%03d\t[%7s]\t%d\t%s\t%.*s"NEWLINE,
            context.time.wYear, context.time.wMonth, context.time.wDay, 
            context.time.wHour, context.time.wMinute, context.time.wSecond, context.time.wMilliseconds,
            severity_name, context.tid, context.func_name, length, text);
        return res;
    }
}

size_t plog::txt_formatter::format(const message& msg, char* buffer, size_t buffer_size) const
{
    size_t len = 0;

    const char* p = msg.first();
    const char* e = msg.last();

    do
    {
        size_t size = 0;
        size_t next = find_eol(p, e, &size);

        int res = format_text_line(buffer + len, buffer_size - len, msg.context(), msg.severity_name(), p, size);
        len = calculate_length(res, len, buffer_size);
        normalize(res, len, buffer);

        p+= next;
    }
    while(p != e);

    return len;
}

size_t plog::txt_formatter::header_size() const
{
    return 0;
}

const char* plog::txt_formatter::header() const
{
    return "";
}
