#include <boost/test/unit_test.hpp>
#include <boost/filesystem.hpp>
#include <boost/regex.hpp>

#include "plog/config.h"
#include "plog/cfq_appender.h"

const std::string msg("All work and no play makes Jack a dull boy\n");

void write_message(plog::appender& ap, int times = 1)
{
    for (int i = 0; i < times; ++i)
    {
        ap.write(msg.data(), msg.length());
    }
}

void remove_test_files()
{
    boost::filesystem::directory_iterator end_itr;
    for(boost::filesystem::directory_iterator i("."); i != end_itr; ++i)
    { 
        if(!boost::filesystem::is_regular_file(i->status())) 
            continue;

        if (boost::regex_match(i->path().filename().string(), boost::regex("shine[0-9]{3}\\.txt")))
        {
            boost::filesystem::remove(i->path());
        }
    }
}

struct fixture
{
    fixture()
    {
        remove_test_files();
    }

    ~fixture()
    {
        remove_test_files();
    }
};

BOOST_FIXTURE_TEST_SUITE(cfq_test, fixture)

BOOST_AUTO_TEST_CASE(test_all)
{
    plog::cfq_appender ap(L"shine", L".txt", 250, 2);

    BOOST_CHECK(boost::filesystem::exists("shine000.txt"));

    write_message(ap, 5);
    BOOST_CHECK(!boost::filesystem::exists("shine001.txt"));

    write_message(ap);
    BOOST_CHECK(boost::filesystem::exists("shine001.txt"));
    BOOST_CHECK_GT(boost::filesystem::file_size("shine001.txt"), 0);

    write_message(ap, 5);
    BOOST_CHECK(!boost::filesystem::exists("shine000.txt"));
    BOOST_CHECK(boost::filesystem::exists("shine002.txt"));
}

BOOST_AUTO_TEST_SUITE_END()