#define BOOST_TEST_MODULE plog_complex_test
#include <boost/test/unit_test.hpp>
#include <boost/filesystem.hpp>
#include <boost/regex.hpp>
#include <fstream>

#include <boost/thread.hpp>
#include <boost/regex.hpp>
#include <boost/range.hpp>
#include "plog/log.h"

namespace 
{
    void test()
    {
        for (int i = 0; i < 10; ++i)
        {
            LOG(plog::info) << "Hello, world!";
            LOG(plog::info) << "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.";
        }
    }

    template<typename F>
    void test_mt(F test_function, int threads)
    {
        boost::thread_group tg;

        for (int i = 0; i < threads; ++i)
        {
            tg.create_thread(test_function);
        }
        tg.join_all();
    }

    void remove_test_files()
    {
        boost::filesystem::directory_iterator end_itr;
        for(boost::filesystem::directory_iterator i("."); i != end_itr; ++i)
        { 
            if(!boost::filesystem::is_regular_file(i->status())) 
                continue;

            if (boost::regex_match(i->path().filename().string(), boost::regex("test-.+\\.(log|csv)")))
            {
                boost::filesystem::remove(i->path());
            }
        }
    }

    struct fixture
    {
        fixture()
        {
            remove_test_files();
        }

        ~fixture()
        {
            plog::free();
            remove_test_files();
        }
    };
}

static const boost::regex verify_txt("\\d+\\.\\d+\\.\\d+\t\\d+:\\d+:\\d+:\\d+\t\\[.{7}]\t\\d+\t[^\t]+\t[^\\[\\]0-9\t]+");
static const boost::regex verify_csv("(\\d+/\\d+/\\d+;\\d+:\\d+:\\d+:\\d+;(TRACE|DEBUG|INFO|WARNING|ERROR|FATAL);\\d+;[^;]+@\\d+;[^;]+;[^;]+;[^;]+|Date;Time;Severity;TID;File;This;Function;Message)");

                                     
#define BOOST_CHECK_LOG_FORMAT(log_name, regexp) \
{\
    std::fstream fs(log_name, std::fstream::in);\
    while (!fs.eof())\
    {\
        char s[1024];\
        fs.getline(s, 1024);\
        if (fs.eof() && !strlen(s))\
            break;\
        boost::smatch match_result;\
        bool matched = boost::regex_match(std::string(s), match_result, regexp);\
        BOOST_CHECK_MESSAGE(matched, s);\
    }\
}

BOOST_FIXTURE_TEST_SUITE(complex_test, fixture)


BOOST_AUTO_TEST_CASE(singlethreaded_txt_log)
{
    plog::init(L"test-st.log", plog::debug, true);
    test();

    BOOST_CHECK_LOG_FORMAT("test-st.log", verify_txt);
}

BOOST_AUTO_TEST_CASE(singlethreaded_csv_log)
{
    plog::init(L"test-st.csv", plog::debug, true);
    test();

    BOOST_CHECK_LOG_FORMAT("test-st.csv", verify_csv);
}

BOOST_AUTO_TEST_CASE(multithreaded_txt_log)
{
    plog::init(L"test-mt.log", plog::debug, true);
    test_mt(test, 10);

    BOOST_CHECK_LOG_FORMAT("test-mt.log", verify_txt);
}

BOOST_AUTO_TEST_CASE(multithreaded_csv_log)
{
    plog::init(L"test-mt.csv", plog::debug, true);
    test_mt(test, 10);

    BOOST_CHECK_LOG_FORMAT("test-mt.csv", verify_csv);
}

BOOST_AUTO_TEST_SUITE_END()
