#define BOOST_TEST_NO_MAIN
#include <boost/test/unit_test.hpp>

#include <boost/filesystem.hpp>

#include "plog/logger_holder.h"
#include "plog/init.h"
#include "plog/log.h"

#include <stdexcept>

namespace
{
    struct fixture_csv
    {
        ~fixture_csv()
        {
            plog::free();
            boost::filesystem::remove("test.csv");
        }
    };

    struct fixture_txt
    {
        ~fixture_txt()
        {
            plog::free();
            boost::filesystem::remove("test.txt");
        }
    };
}

BOOST_FIXTURE_TEST_SUITE(default_initializer_test, fixture_csv)

BOOST_AUTO_TEST_CASE(initialize_uninitialize_log)
{
    plog::init(L"test.csv", plog::debug);
    BOOST_REQUIRE_NE(plog::logger_holder::get_logger(), (plog::logger*)NULL);
    BOOST_REQUIRE(boost::filesystem::exists("test.csv"));

    plog::free( true );
    BOOST_REQUIRE_EQUAL( plog::logger_holder::get_logger(), (plog::logger*)NULL);
}

BOOST_AUTO_TEST_CASE(double_initialize_log)
{
    plog::init(L"test.csv", plog::debug);
    BOOST_REQUIRE_NE(plog::logger_holder::get_logger(), (plog::logger*)NULL);

    BOOST_REQUIRE_THROW( plog::init(L"smth.csv", plog::debug), std::runtime_error );
    
    plog::free( true );
    BOOST_REQUIRE_EQUAL( plog::logger_holder::get_logger(), (plog::logger*)NULL);
}

BOOST_AUTO_TEST_CASE(double_uninitialize_log)
{
    plog::init(L"test.csv", plog::debug);
    BOOST_REQUIRE_NE(plog::logger_holder::get_logger(), (plog::logger*)NULL);

    plog::free( true );
    BOOST_REQUIRE_EQUAL( plog::logger_holder::get_logger(), (plog::logger*)NULL);

    BOOST_REQUIRE_NO_THROW( plog::free( true ));
}

BOOST_AUTO_TEST_CASE(double_using_uninitialized_log)
{
    BOOST_REQUIRE_NO_THROW(LOG(plog::info) << "Oh my God, they killed Kenny!");
}

BOOST_AUTO_TEST_SUITE_END()

BOOST_FIXTURE_TEST_SUITE(txt_initializer_test, fixture_txt)

BOOST_AUTO_TEST_CASE(initialize_uninitialize_log)
{
    plog::init_txt(L"test.txt", plog::debug);
    BOOST_REQUIRE_NE(plog::logger_holder::get_logger(), (plog::logger*)NULL);
    BOOST_REQUIRE(boost::filesystem::exists("test.txt"));

    plog::free( true );
    BOOST_REQUIRE_EQUAL( plog::logger_holder::get_logger(), (plog::logger*)NULL);
}

BOOST_AUTO_TEST_SUITE_END()

BOOST_AUTO_TEST_SUITE(ostream_initializer_test)

BOOST_AUTO_TEST_CASE(initialize_uninitialize_log)
{
    std::stringstream s;
    plog::init_ostream(s, plog::debug);
    BOOST_REQUIRE_NE(plog::logger_holder::get_logger(), (plog::logger*)NULL);
    
    LOG(plog::info) << "Hello world";

    BOOST_REQUIRE_NE(std::string::npos, s.str().find("Hello world"));

    plog::free( true );
    BOOST_REQUIRE_EQUAL(plog::logger_holder::get_logger(), (plog::logger*)NULL);
}

BOOST_AUTO_TEST_SUITE_END()
