#define BOOST_TEST_NO_MAIN
#include <boost/test/unit_test.hpp>
#include "plog/init.h"
#include "test_utils.h"

BOOST_AUTO_TEST_SUITE(parse_filename)

BOOST_AUTO_TEST_CASE(parse_log)
{
    std::wstring base, extension;
    plog::detail::parse_file_name(L"file.log", base, extension);

    BOOST_CHECK_EQUAL(L"file", base);
    BOOST_CHECK_EQUAL(L".log", extension);
}

BOOST_AUTO_TEST_CASE(parse_csv)
{
    std::wstring base, extension;
    plog::detail::parse_file_name(L"file.csv", base, extension);

    BOOST_CHECK_EQUAL(L"file", base);
    BOOST_CHECK_EQUAL(L".csv", extension);
}

BOOST_AUTO_TEST_CASE(parse_txt)
{
    std::wstring base, extension;
    plog::detail::parse_file_name(L"file.txt", base, extension);

    BOOST_CHECK_EQUAL(L"file", base);
    BOOST_CHECK_EQUAL(L".txt", extension);
}

BOOST_AUTO_TEST_CASE(parse_with_folder_win)
{
    std::wstring base, extension;
    plog::detail::parse_file_name(L"C:\\Program Files\\file.csv", base, extension);

    BOOST_CHECK_EQUAL(L"C:\\Program Files\\file", base);
    BOOST_CHECK_EQUAL(L".csv", extension);
}

BOOST_AUTO_TEST_CASE(parse_with_folder_nix)
{
    std::wstring base, extension;
    plog::detail::parse_file_name(L"/home/mayhem/file.csv", base, extension);

    BOOST_CHECK_EQUAL(L"/home/mayhem/file", base);
    BOOST_CHECK_EQUAL(L".csv", extension);
}

BOOST_AUTO_TEST_CASE(parse_with_folder_dots)
{
    std::wstring base, extension;
    plog::detail::parse_file_name(L"/home/tyler.durden/file.csv", base, extension);

    BOOST_CHECK_EQUAL(L"/home/tyler.durden/file", base);
    BOOST_CHECK_EQUAL(L".csv", extension);
}

BOOST_AUTO_TEST_CASE(parse_last_dot)
{
    std::wstring base, extension;
    plog::detail::parse_file_name(L"/home/tyler.durden/file.", base, extension);

    BOOST_CHECK_EQUAL(L"/home/tyler.durden/file", base);
    BOOST_CHECK_EQUAL(L".", extension);
}

BOOST_AUTO_TEST_CASE(parse_noext)
{
    std::wstring base, extension;
    plog::detail::parse_file_name(L"/home/tyler.durden/file", base, extension);

    BOOST_CHECK_EQUAL(L"/home/tyler.durden/file", base);
    BOOST_CHECK_EQUAL(L"", extension);
}

BOOST_AUTO_TEST_CASE(parse_first_dot)
{
    std::wstring base, extension;
    plog::detail::parse_file_name(L".file", base, extension);

    BOOST_CHECK_EQUAL(L".file", base);
    BOOST_CHECK_EQUAL(L"", extension);
}

BOOST_AUTO_TEST_CASE(parse_first_dot_with_folder)
{
    std::wstring base, extension;
    plog::detail::parse_file_name(L"/home/tyler.durden/.file", base, extension);

    BOOST_CHECK_EQUAL(L"/home/tyler.durden/.file", base);
    BOOST_CHECK_EQUAL(L"", extension);
}

BOOST_AUTO_TEST_SUITE_END()

BOOST_AUTO_TEST_SUITE(is_csv)

BOOST_AUTO_TEST_CASE(log)
{
    BOOST_CHECK_EQUAL(plog::detail::is_csv(L"file.log"), false);
}

BOOST_AUTO_TEST_CASE(csv)
{
    BOOST_CHECK_EQUAL(plog::detail::is_csv(L"file.csv"), true);
}

BOOST_AUTO_TEST_CASE(something_completely_different)
{
    BOOST_CHECK_EQUAL(plog::detail::is_csv(L"file.000"), false);
}

BOOST_AUTO_TEST_CASE(noext)
{
    BOOST_CHECK_EQUAL(plog::detail::is_csv(L"file"), false);
}

BOOST_AUTO_TEST_CASE(too_small)
{
    BOOST_CHECK_EQUAL(plog::detail::is_csv(L"xz"), false);
}

BOOST_AUTO_TEST_SUITE_END()
