#include <boost/test/unit_test.hpp>

#include "plog/log.h"
#include "plog/raw_formatter.h"
#include <iostream>
namespace
{
    struct fixture 
    {
        ~fixture()         
        { 
            plog::free(); 
        }
    };
}
BOOST_FIXTURE_TEST_SUITE(log_macros_test, fixture)

BOOST_AUTO_TEST_CASE(test_every_n)
{
    std::string expected = "4\n9\n14\n";
    std::stringstream actual;
    plog::init_ostream(actual, plog::info, new plog::raw_formatter);

    for (int i = 0; i <= 16; ++i)
    {
        LOG_EVERY_N(plog::info, 5) << (int)i;
    }

    BOOST_REQUIRE(expected == actual.str());
}

BOOST_AUTO_TEST_CASE(test_if_every_n)
{
    std::string expected = "9\n14\n";
    std::stringstream actual;
    plog::init_ostream(actual, plog::info, new plog::raw_formatter);

    for (int i = 0; i <= 16; ++i)
    {
        LOG_IF_EVERY_N(plog::info, i > 7, 5) << i;
    }

    BOOST_REQUIRE(expected == actual.str());
}

BOOST_AUTO_TEST_CASE(test_first_n)
{
    std::string expected = "0\n1\n2\n3\n4\n";
    std::stringstream actual;
    plog::init_ostream(actual, plog::info, new plog::raw_formatter);

    for (int i = 0; i <= 16; ++i)
    {
        LOG_FIRST_N(plog::info, 5) << i;
    }

    BOOST_REQUIRE(expected == actual.str());
}

BOOST_AUTO_TEST_SUITE_END()
