#include <boost/test/unit_test.hpp>

#include "plog/raw_formatter.h"


namespace
{
    const char short_msg[] = "short message is short";

    const char long_msg[] = 
        "long message is so \
looooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooong";
}

static const size_t BUF_SIZE = 512;
static char buffer[BUF_SIZE];
static const std::locale loc("");

BOOST_AUTO_TEST_SUITE(raw_formatter)

BOOST_AUTO_TEST_CASE(format_short_message)
{
    SYSTEMTIME time = {1986, 9, 1, 1, 12, 0, 0, 0};
    plog::message msg(
        plog::info, 
        plog::context_info(__FILE__, __FUNCTION__, 777, 1234, NULL, &time), 
        loc,
        short_msg);

    plog::raw_formatter formatter;
    formatter.format(msg, buffer, BUF_SIZE);

    std::stringstream expected;
    expected << short_msg << "\n";

    BOOST_REQUIRE_EQUAL(expected.str().length(), strlen(buffer));
    BOOST_REQUIRE_EQUAL(expected.str(), buffer);
}

BOOST_AUTO_TEST_CASE(format_long_message)
{
    SYSTEMTIME time = {1986, 9, 1, 1, 12, 0, 0, 0};
    plog::message msg(
        plog::info, 
        plog::context_info(__FILE__, __FUNCTION__, 0, 0, 0, &time), 
        loc,
        long_msg);

    plog::raw_formatter formatter;
    size_t length = formatter.format(msg, buffer, BUF_SIZE);

    std::stringstream expected;
    expected << long_msg << "\n";

    BOOST_REQUIRE_EQUAL(static_cast<size_t>(511), strlen(buffer));
    BOOST_REQUIRE_EQUAL(expected.str().substr(0, 511), buffer);
    BOOST_CHECK_EQUAL(size_t(511), length);
}

BOOST_AUTO_TEST_SUITE_END()