#include <boost/test/unit_test.hpp>

#include "plog/simple_formatter.h"


namespace
{
    const char short_msg[] = "short message is short";

    const char long_msg[] = 
        "long message is so \
        looooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
        looooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
        looooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
        looooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
        looooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
        looooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
        looooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo\
        looooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooong";
}

BOOST_AUTO_TEST_SUITE(simple_formatter)

BOOST_AUTO_TEST_CASE(format_short_message)
{
    SYSTEMTIME time = {1986, 9, 1, 1, 12, 0, 0, 0};
    plog::message msg(
        plog::info, 
        plog::context_info(__FILE__, __FUNCTION__, 777, 1234, NULL, &time), 
        short_msg);

    plog::simple_formatter formatter;
    formatter.format(msg);

    std::stringstream expected;
    expected << short_msg << "\n";

    BOOST_REQUIRE_EQUAL(expected.str().length(), strlen(formatter.buffer()));
    BOOST_REQUIRE_EQUAL(expected.str(), formatter.buffer());
}

BOOST_AUTO_TEST_CASE(format_long_message)
{
    SYSTEMTIME time = {1986, 9, 1, 1, 12, 0, 0, 0};
    plog::message msg(
        plog::info, 
        plog::context_info(__FILE__, __FUNCTION__, 777, 1234, NULL, &time), 
        long_msg);

    plog::simple_formatter formatter;
    formatter.format(msg);

    std::stringstream expected;
    expected << long_msg << "\n";

    BOOST_REQUIRE_EQUAL(static_cast<size_t>(511), strlen(formatter.buffer()));
    BOOST_REQUIRE_EQUAL(expected.str().substr(0, 511), formatter.buffer());
}

BOOST_AUTO_TEST_SUITE_END()